@@ -7,22 +7,45 @@
 #include "pinyindictionary.h"
 #include "constants.h"
 #include "libime/core/datrie.h"
+#include "libime/core/dictionary.h"
+#include "libime/core/languagemodel.h"
 #include "libime/core/lattice.h"
 #include "libime/core/lrucache.h"
+#include "libime/core/segmentgraph.h"
 #include "libime/core/triedictionary.h"
 #include "libime/core/utils.h"
+#include "libime/core/utils_p.h"
 #include "libime/core/zstdfilter.h"
+#include "libime/pinyin/pinyinmatchstate.h"
 #include "pinyindecoder_p.h"
 #include "pinyinencoder.h"
 #include "pinyinmatchstate_p.h"
-#include <boost/algorithm/string.hpp>
+#include <algorithm>
+#include <boost/container_hash/hash.hpp>
+#include <cassert>
 #include <cmath>
+#include <cstddef>
+#include <cstdint>
+#include <fcitx-utils/macros.h>
+#include <fcitx-utils/signals.h>
+#include <fcitx-utils/stringutils.h>
 #include <fstream>
 #include <iomanip>
+#include <ios>
+#include <istream>
+#include <iterator>
+#include <list>
+#include <memory>
 #include <optional>
+#include <ostream>
 #include <queue>
+#include <stdexcept>
 #include <string>
 #include <string_view>
+#include <tuple>
+#include <unordered_set>
+#include <utility>
+#include <vector>
 
 namespace libime {
 
@@ -143,7 +166,7 @@ const SegmentGraphNode *prevIsSeparator(const SegmentGraph &graph,
         const auto range = node.prevs();
         const auto &prev = range.front();
         auto pinyin = graph.segment(prev, node);
-        if (boost::starts_with(pinyin, "\'")) {
+        if (pinyin.starts_with("\'")) {
             return &prev;
         }
     }
@@ -158,10 +181,10 @@ inline void searchOneStep(
     auto iter = nodes.begin();
     while (iter != nodes.end()) {
         if (current != 0) {
-            PinyinTrie::value_type result;
-            result = iter->first->traverse(&current, 1, iter->second);
+            const auto resultRaw =
+                iter->first->traverseRaw(&current, 1, iter->second);
 
-            if (PinyinTrie::isNoPath(result)) {
+            if (PinyinTrie::isNoPathRaw(resultRaw)) {
                 nodes.erase(iter++);
             } else {
                 iter++;
@@ -171,8 +194,8 @@ inline void searchOneStep(
             for (char test = PinyinEncoder::firstFinal;
                  test <= PinyinEncoder::lastFinal; test++) {
                 decltype(extraNodes)::value_type p = *iter;
-                auto result = p.first->traverse(&test, 1, p.second);
-                if (!PinyinTrie::isNoPath(result)) {
+                const auto resultRaw = p.first->traverseRaw(&test, 1, p.second);
+                if (!PinyinTrie::isNoPathRaw(resultRaw)) {
                     extraNodes.push_back(p);
                     changed = true;
                 }
@@ -195,6 +218,10 @@ size_t fuzzyFactor(PinyinFuzzyFlags flags) {
         flags = flags.unset(PinyinFuzzyFlag::Correction);
         factor += PINYIN_CORRECTION_FUZZY_FACTOR;
     }
+    if (flags.test(PinyinFuzzyFlag::AdvancedTypo)) {
+        flags = flags.unset(PinyinFuzzyFlag::AdvancedTypo);
+        factor += PINYIN_ADVACNED_TYPO_FUZZY_FACTOR;
+    }
     if (flags != 0) {
         factor += 1;
     }
@@ -204,16 +231,15 @@ size_t fuzzyFactor(PinyinFuzzyFlags flags) {
 PinyinDictionary::TrieType loadTextImpl(std::istream &in) {
     PinyinDictionary::TrieType trie;
 
-    std::string buf;
-    auto isSpaceCheck = boost::is_any_of(" \n\t\r\v\f");
+    std::string lineBuf;
     while (!in.eof()) {
-        if (!std::getline(in, buf)) {
+        if (!std::getline(in, lineBuf)) {
             break;
         }
 
-        boost::trim_if(buf, isSpaceCheck);
-        std::vector<std::string> tokens;
-        boost::split(tokens, buf, isSpaceCheck);
+        auto line = fcitx::stringutils::trimView(lineBuf);
+        std::vector<std::string> tokens =
+            fcitx::stringutils::split(line, FCITX_WHITESPACE);
         if (tokens.size() == 3 || tokens.size() == 2) {
             const std::string &hanzi = tokens[0];
             std::string_view pinyin = tokens[1];
@@ -230,7 +256,7 @@ PinyinDictionary::TrieType loadTextImpl(std::istream &in) {
                 trie.set(result.data(), result.size(), prob);
             } catch (const std::invalid_argument &e) {
                 LIBIME_ERROR()
-                    << "Failed to parse line: " << buf << ", skipping.";
+                    << "Failed to parse line: " << line << ", skipping.";
             }
         }
     }
@@ -283,8 +309,7 @@ public:
         const std::unordered_set<const SegmentGraphNode *> &ignore,
         NodeToMatchedPinyinPathsMap &matchedPaths)
         : graph_(graph), hasher_(graph), callback_(callback), ignore_(ignore),
-          matchedPathsMap_(&matchedPaths), nodeCacheMap_(nullptr),
-          matchCacheMap_(nullptr) {}
+          matchedPathsMap_(&matchedPaths) {}
 
     PinyinMatchContext(const PinyinMatchContext &) = delete;
 
@@ -335,9 +360,8 @@ void PinyinDictionaryPrivate::addEmptyMatch(
     const SegmentGraph &graph = context.graph_;
     // Create a new starting point for current node, and put it in matchResult.
     if (&currentNode != &graph.end() &&
-        !boost::starts_with(
-            graph.segment(currentNode.index(), currentNode.index() + 1),
-            "\'")) {
+        !graph.segment(currentNode.index(), currentNode.index() + 1)
+             .starts_with("\'")) {
         SegmentGraphPath vec;
         if (const auto *prev = prevIsSeparator(graph, currentNode)) {
             vec.push_back(prev);
@@ -371,8 +395,8 @@ PinyinTriePositions traverseAlongPathOneStepBySyllables(
             // make a copy
             auto pos = _pos;
             auto initial = static_cast<char>(syl.first);
-            auto result = path.trie()->traverse(&initial, 1, pos);
-            if (PinyinTrie::isNoPath(result)) {
+            const auto resultRaw = path.trie()->traverseRaw(&initial, 1, pos);
+            if (PinyinTrie::isNoPathRaw(resultRaw)) {
                 continue;
             }
             const auto &finals = syl.second;
@@ -381,9 +405,9 @@ PinyinTriePositions traverseAlongPathOneStepBySyllables(
                                                            size_t fuzzyFactor,
                                                            auto pos) {
                 auto final = static_cast<char>(pyFinal);
-                auto result = path.trie()->traverse(&final, 1, pos);
+                const auto resultRaw = path.trie()->traverseRaw(&final, 1, pos);
 
-                if (!PinyinTrie::isNoPath(result)) {
+                if (!PinyinTrie::isNoPathRaw(resultRaw)) {
                     size_t newFuzzies = fuzzies + fuzzyFactor;
                     positions.emplace_back(pos, newFuzzies);
                 }
@@ -420,8 +444,8 @@ void matchWordsOnTrie(const PinyinTrie *userDict, const MatchedPinyinPath &path,
                 [userDict, &path, &callback, extraCost, isCorrection](
                     PinyinTrie::value_type value, size_t len, uint64_t pos) {
                     std::string s;
-                    s.reserve(len + path.size() * 2);
-                    path.trie()->suffix(s, len + path.size() * 2, pos);
+                    s.reserve(len + (path.size() * 2));
+                    path.trie()->suffix(s, len + (path.size() * 2), pos);
                     if (size_t separator =
                             s.find(pinyinHanziSep, path.size() * 2);
                         separator != std::string::npos) {
@@ -429,7 +453,7 @@ void matchWordsOnTrie(const PinyinTrie *userDict, const MatchedPinyinPath &path,
                         auto encodedPinyin = view.substr(0, separator);
                         auto hanzi = view.substr(separator + 1);
                         const size_t lengthDiff =
-                            (encodedPinyin.size() / 2 - path.size());
+                            ((encodedPinyin.size() / 2) - path.size());
                         // Don't match long word for "custom".
                         if (path.trie() == userDict && value < 0 &&
                             lengthDiff > 0) {
@@ -446,8 +470,8 @@ void matchWordsOnTrie(const PinyinTrie *userDict, const MatchedPinyinPath &path,
                 pos);
         } else {
             const char sep = pinyinHanziSep;
-            auto result = path.trie()->traverse(&sep, 1, pos);
-            if (PinyinTrie::isNoPath(result)) {
+            const auto resultRaw = path.trie()->traverseRaw(&sep, 1, pos);
+            if (PinyinTrie::isNoPathRaw(resultRaw)) {
                 continue;
             }
 
@@ -455,11 +479,11 @@ void matchWordsOnTrie(const PinyinTrie *userDict, const MatchedPinyinPath &path,
                 [&path, &callback, extraCost, isCorrection](
                     PinyinTrie::value_type value, size_t len, uint64_t pos) {
                     std::string s;
-                    s.reserve(len + path.size() * 2 + 1);
-                    path.trie()->suffix(s, len + path.size() * 2 + 1, pos);
+                    s.reserve(len + (path.size() * 2) + 1);
+                    path.trie()->suffix(s, len + (path.size() * 2) + 1, pos);
                     std::string_view view(s);
                     auto encodedPinyin = view.substr(0, path.size() * 2);
-                    auto hanzi = view.substr(path.size() * 2 + 1);
+                    auto hanzi = view.substr((path.size() * 2) + 1);
                     callback(encodedPinyin, hanzi, value + extraCost,
                              isCorrection);
                     return true;
@@ -564,7 +588,7 @@ void PinyinDictionaryPrivate::findMatchesBetween(
     auto pinyin = graph.segment(prevNode, currentNode);
     // If predecessor is a separator, just copy every existing match result
     // over and don't traverse on the trie.
-    if (boost::starts_with(pinyin, "\'")) {
+    if (pinyin.starts_with("\'")) {
         const auto &prevMatches = matchedPathsMap[&prevNode];
         for (const auto &match : prevMatches) {
             // copy the path, and append current node.
@@ -629,7 +653,7 @@ void PinyinDictionaryPrivate::findMatchesBetween(
         }
     }
 
-    if (!context.ignore_.count(&currentNode)) {
+    if (!context.ignore_.contains(&currentNode)) {
         // after we match current syllable, we first try to match word.
         if (!matchWords(context, newPaths)) {
             // If we failed to match any length 1 word, add a new empty word
@@ -656,7 +680,7 @@ void PinyinDictionaryPrivate::matchNode(
     const SegmentGraphNode &currentNode) const {
     auto &matchedPathsMap = *context.matchedPathsMap_;
     // Check if the node has been searched already.
-    if (matchedPathsMap.count(&currentNode)) {
+    if (matchedPathsMap.contains(&currentNode)) {
         return;
     }
     auto &currentMatches = matchedPathsMap[&currentNode];
@@ -876,7 +900,7 @@ void PinyinDictionary::saveText(size_t idx, std::ostream &out) {
         std::string_view ref(buf);
         auto fullPinyin = PinyinEncoder::decodeFullPinyin(ref.data(), sep);
         out << ref.substr(sep + 1) << " " << fullPinyin << " "
-            << std::setprecision(16) << value << std::endl;
+            << std::setprecision(16) << value << '\n';
         return true;
     });
     out.copyfmt(state);
